import 'dart:ui';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';
import 'package:smart_tab_layout/src/smart_underline_tab_indicator.dart';
import 'package:smart_tab_layout/src/smart_image_tab_indicator.dart';

/// 指示器
class _IndicatorPainter extends CustomPainter {
  _IndicatorPainter({
    required this.controller,
    required this.direction,
    required this.indicator,
    required this.indicatorSize,
    required this.indicatorWidth,
    required this.tabKeys,
    required _IndicatorPainter? old,
    required this.indicatorPadding,
  })  : assert(controller != null),
        assert(indicator != null),
        super(repaint: controller.animation) {
    if (old != null)
      saveTabOffsets(old._currentTabOffsets, old._currentTextDirection);
  }

  final TabController controller;
  final Axis direction;
  final Decoration indicator;
  final TabBarIndicatorSize? indicatorSize;
  final double? indicatorWidth;
  final EdgeInsetsGeometry indicatorPadding;
  final List<GlobalKey> tabKeys;

  // _currentTabOffsets and _currentTextDirection are set each time TabBar
  // layout is completed. These values can be null when TabBar contains no
  // tabs, since there are nothing to lay out.
  List<double>? _currentTabOffsets;
  TextDirection? _currentTextDirection;

  Rect? _currentRect;
  BoxPainter? _painter;
  bool _needsPaint = false;

  void markNeedsPaint() {
    _needsPaint = true;
  }

  void dispose() {
    _painter?.dispose();
  }

  void saveTabOffsets(List<double>? tabOffsets, TextDirection? textDirection) {
    _currentTabOffsets = tabOffsets;
    _currentTextDirection = textDirection;
  }

  // _currentTabOffsets[index] is the offset of the start edge of the tab at index, and
  // _currentTabOffsets[_currentTabOffsets.length] is the end edge of the last tab.
  int get maxTabIndex => _currentTabOffsets!.length - 2;

  double centerOf(int tabIndex) {
    assert(_currentTabOffsets != null);
    assert(_currentTabOffsets!.isNotEmpty);
    assert(tabIndex >= 0);
    assert(tabIndex <= maxTabIndex);
    tabIndex = tabIndex * 2;
    return (_currentTabOffsets![tabIndex] + _currentTabOffsets![tabIndex + 1]) /
        2.0;
  }

  Rect indicatorRect(Size tabBarSize, int oldTabIndex) {
    assert(_currentTabOffsets != null);
    assert(_currentTextDirection != null);
    assert(_currentTabOffsets!.isNotEmpty);
    assert(oldTabIndex >= 0);
    assert(oldTabIndex <= maxTabIndex);
    int tabIndex = oldTabIndex * 2;
    double tabLeft, tabRight;
    switch (_currentTextDirection!) {
      case TextDirection.rtl:
        tabLeft = _currentTabOffsets![tabIndex + 2];
        tabRight = _currentTabOffsets![tabIndex];
        break;
      case TextDirection.ltr:
        tabLeft = _currentTabOffsets![tabIndex];
        tabRight = _currentTabOffsets![tabIndex + 1];
        break;
    }

    if (indicatorWidth != null) {
      final double tabWidth = indicatorWidth!;
      final double delta = ((tabRight - tabLeft) - tabWidth) / 2.0;
      tabLeft += delta;
      tabRight -= delta;
    } else if (indicatorSize == TabBarIndicatorSize.label) {
      final double tabWidth = direction == Axis.horizontal
          ? tabKeys[oldTabIndex].currentContext!.size!.width
          : tabKeys[oldTabIndex].currentContext!.size!.height;
      final double delta = ((tabRight - tabLeft) - tabWidth) / 2.0;
      tabLeft += delta;
      tabRight -= delta;
    }

    final EdgeInsets insets = indicatorPadding.resolve(_currentTextDirection);
    final Rect rect;
    if (direction == Axis.horizontal) {
      rect = Rect.fromLTWH(tabLeft, 0.0, tabRight - tabLeft, tabBarSize.height);
    } else {
      rect = Rect.fromLTWH(0.0, tabLeft, tabBarSize.width, tabRight - tabLeft);
    }

    if (!(rect.size >= insets.collapsedSize)) {
      throw FlutterError(
        'indicatorPadding insets should be less than Tab Size\n'
        'Rect Size : ${rect.size}, Insets: ${insets.toString()}',
      );
    }
    return insets.deflateRect(rect);
  }

  @override
  void paint(Canvas canvas, Size size) {
    _needsPaint = false;
    _painter ??= indicator.createBoxPainter(markNeedsPaint);

    final double index = controller.index.toDouble();
    final double value = controller.animation!.value;
    final bool ltr = index > value;
    final int from =
        (ltr ? value.floor() : value.ceil()).clamp(0, maxTabIndex).toInt();
    final int to = (ltr ? from + 1 : from - 1).clamp(0, maxTabIndex).toInt();
    final Rect fromRect = indicatorRect(size, from);
    final Rect toRect = indicatorRect(size, to);
    _currentRect = Rect.lerp(fromRect, toRect, (value - from).abs());
    assert(_currentRect != null);

    final ImageConfiguration configuration = ImageConfiguration(
      size: _currentRect!.size,
      textDirection: _currentTextDirection,
    );
    _painter!.paint(canvas, _currentRect!.topLeft, configuration);
  }

  @override
  bool shouldRepaint(_IndicatorPainter old) {
    return _needsPaint ||
        controller != old.controller ||
        indicator != old.indicator ||
        tabKeys.length != old.tabKeys.length ||
        (!listEquals(_currentTabOffsets, old._currentTabOffsets)) ||
        _currentTextDirection != old._currentTextDirection;
  }
}

typedef _LayoutCallback = void Function(
    List<double> xOffsets, TextDirection textDirection, double width);

// This class and its renderer class only exist to report the widths of the tabs
// upon layout. The tab widths are only used at paint time (see _IndicatorPainter)
// or in response to input.
class _TabLabelBar extends Flex {
  _TabLabelBar({
    Key? key,
    Axis direction = Axis.horizontal,
    List<Widget> children = const <Widget>[],
    required this.onPerformLayout,
  }) : super(
          key: key,
          children: children,
          direction: direction,
          mainAxisSize: MainAxisSize.max,
          mainAxisAlignment: MainAxisAlignment.start,
          crossAxisAlignment: CrossAxisAlignment.center,
          verticalDirection: VerticalDirection.down,
          textDirection: TextDirection.ltr,
        );

  final _LayoutCallback onPerformLayout;

  @override
  RenderFlex createRenderObject(BuildContext context) {
    return _TabLabelBarRenderer(
      direction: direction,
      mainAxisAlignment: mainAxisAlignment,
      mainAxisSize: mainAxisSize,
      crossAxisAlignment: crossAxisAlignment,
      textDirection: getEffectiveTextDirection(context)!,
      verticalDirection: verticalDirection,
      onPerformLayout: onPerformLayout,
    );
  }

  @override
  void updateRenderObject(
      BuildContext context, _TabLabelBarRenderer renderObject) {
    super.updateRenderObject(context, renderObject);
    renderObject.onPerformLayout = onPerformLayout;
  }
}

class _TabLabelBarRenderer extends RenderFlex {
  _TabLabelBarRenderer({
    List<RenderBox>? children,
    required Axis direction,
    required MainAxisSize mainAxisSize,
    required MainAxisAlignment mainAxisAlignment,
    required CrossAxisAlignment crossAxisAlignment,
    required TextDirection textDirection,
    required VerticalDirection verticalDirection,
    required this.onPerformLayout,
  })  : assert(onPerformLayout != null),
        assert(textDirection != null),
        super(
          children: children,
          direction: direction,
          mainAxisSize: mainAxisSize,
          mainAxisAlignment: mainAxisAlignment,
          crossAxisAlignment: crossAxisAlignment,
          textDirection: textDirection,
          verticalDirection: verticalDirection,
        );

  _LayoutCallback onPerformLayout;

  @override
  void performLayout() {
    super.performLayout();
    // xOffsets will contain childCount+1 values, giving the offsets of the
    // leading edge of the first tab as the first value, of the leading edge of
    // the each subsequent tab as each subsequent value, and of the trailing
    // edge of the last tab as the last value.
    RenderBox? child = firstChild;
    final List<double> xOffsets = <double>[];
    while (child != null) {
      final FlexParentData childParentData =
          child.parentData! as FlexParentData;
      xOffsets.add(direction == Axis.horizontal
          ? childParentData.offset.dx
          : childParentData.offset.dy);
      assert(child.parentData == childParentData);
      child = childParentData.nextSibling;
    }
    assert(textDirection != null);
    double width = direction == Axis.horizontal ? size.width : size.height;
    switch (textDirection!) {
      case TextDirection.rtl:
        xOffsets.insert(0, width);
        break;
      case TextDirection.ltr:
        xOffsets.add(width);
        break;
    }
    onPerformLayout(xOffsets, textDirection!, width);
  }
}

// This class, and TabBarScrollPosition, only exist to handle the case
// where a scrollable TabBar has a non-zero initialIndex.
class _TabBarScrollController extends ScrollController {
  _TabBarScrollController(this.tabBar);

  final _SmartTabLayoutState tabBar;

  @override
  ScrollPosition createScrollPosition(ScrollPhysics physics,
      ScrollContext context, ScrollPosition? oldPosition) {
    return _TabBarScrollPosition(
      physics: physics,
      context: context,
      oldPosition: oldPosition,
      tabBar: tabBar,
    );
  }
}

// This class, and TabBarScrollController, only exist to handle the case
// where a scrollable TabBar has a non-zero initialIndex. In that case we can
// only compute the scroll position's initial scroll offset (the "correct"
// pixels value) after the TabBar viewport width and scroll limits are known.
class _TabBarScrollPosition extends ScrollPositionWithSingleContext {
  _TabBarScrollPosition({
    required ScrollPhysics physics,
    required ScrollContext context,
    required ScrollPosition? oldPosition,
    required this.tabBar,
  }) : super(
          physics: physics,
          context: context,
          initialPixels: null,
          oldPosition: oldPosition,
        );

  final _SmartTabLayoutState tabBar;

  bool? _initialViewportDimensionWasZero;

  @override
  bool applyContentDimensions(double minScrollExtent, double maxScrollExtent) {
    bool result = true;
    if (_initialViewportDimensionWasZero != true) {
      // If the viewport never had a non-zero dimension, we just want to jump
      // to the initial scroll position to avoid strange scrolling effects in
      // release mode: In release mode, the viewport temporarily may have a
      // dimension of zero before the actual dimension is calculated. In that
      // scenario, setting the actual dimension would cause a strange scroll
      // effect without this guard because the super call below would starts a
      // ballistic scroll activity.
      assert(viewportDimension != null);
      _initialViewportDimensionWasZero = viewportDimension != 0.0;
      correctPixels(tabBar._initialScrollOffset(
          viewportDimension, minScrollExtent, maxScrollExtent));
      result = false;
    }
    return super.applyContentDimensions(minScrollExtent, maxScrollExtent) &&
        result;
  }
}

/// tab样式
class _TabStyle extends AnimatedWidget {
  const _TabStyle({
    Key? key,
    required Animation<double> animation,
    required this.selected,
    required this.labelStyle,
    required this.selectedLabelStyle,
    required this.child,
    this.index,
    this.widget,
  }) : super(key: key, listenable: animation);

  final TextStyle labelStyle;
  final TextStyle selectedLabelStyle;
  final bool selected;
  final Widget child;
  final SmartTabLayout? widget;
  final int? index;

  @override
  Widget build(BuildContext context) {
    final Animation<double> animation = listenable as Animation<double>;

    // To enable TextStyle.lerp(style1, style2, value), both styles must have
    // the same value of inherit. Force that to be inherit=true here.
    final TextStyle textStyle = selected
        ? TextStyle.lerp(selectedLabelStyle, labelStyle, animation.value)!
        : TextStyle.lerp(labelStyle, selectedLabelStyle, animation.value)!;
    final double? fontSize =
        selected ? selectedLabelStyle.fontSize : labelStyle.fontSize;
    Widget child = this.child;
    if (widget != null) {
      child = _buildStyledTab(child, animation);
    }

    return DefaultTextStyle(
      softWrap: false,
      style: textStyle.copyWith(fontSize: fontSize, fontFamily: "PingFang SC"),
      maxLines: 1,
      overflow: TextOverflow.ellipsis,
      child: child,
    );
  }

  _buildStyledTab(Widget child, Animation<double> animation) {
    double? width = selected
        ? lerpDouble(
            widget?.itemSelectedWidth, widget?.itemWidth, animation.value)
        : lerpDouble(
            widget?.itemWidth, widget?.itemSelectedWidth, animation.value);

    double? height = selected
        ? lerpDouble(
            widget?.itemSelectedHeight, widget?.itemHeight, animation.value)
        : lerpDouble(
            widget?.itemHeight, widget?.itemSelectedHeight, animation.value);

    EdgeInsets? padding = selected
        ? EdgeInsets.lerp(
            widget?.itemSelectedPadding, widget?.itemPadding, animation.value)
        : EdgeInsets.lerp(
            widget?.itemPadding, widget?.itemSelectedPadding, animation.value);

    Color? background = selected
        ? Color.lerp(widget?.itemSelectedBackground, widget?.itemBackground,
            animation.value)
        : Color.lerp(widget?.itemBackground, widget?.itemSelectedBackground,
            animation.value);

    BoxBorder? border = selected
        ? BoxBorder.lerp(
            widget?.itemSelectedBorder, widget?.itemBorder, animation.value)
        : BoxBorder.lerp(
            widget?.itemBorder, widget?.itemSelectedBorder, animation.value);

    BorderRadius? borderRadius = selected
        ? BorderRadius.lerp(widget?.itemSelectedBorderRadius,
            widget?.itemBorderRadius, animation.value)
        : BorderRadius.lerp(widget?.itemBorderRadius,
            widget?.itemSelectedBorderRadius, animation.value);

    return SmartTab(
      width: width,
      height: height,
      padding: padding,
      background: background,
      isInkWell: widget!.isInkWell,
      splashColor: widget!.itemSplashColor,
      shadowColor: widget?.itemShadowColor,
      elevation: widget!.itemElevation,
      border: border,
      borderRadius: borderRadius,
      child: child,
      onTap: () {
        widget!.controller!.animateTo(index!);
        widget?.onTap?.call(index!);
      },
    );
  }
}

/// tab子项
class SmartTab extends StatelessWidget {
  final Widget child;
  final double? height;
  final double? width;
  final EdgeInsets? padding;
  final Color? background;
  final Color splashColor;
  final Color? shadowColor;
  final double elevation;
  final BorderRadius? borderRadius;
  final BoxBorder? border;
  final GestureTapCallback onTap;
  final bool isInkWell;

  SmartTab({
    Key? key,
    required this.child,
    this.height,
    this.width,
    this.padding,
    this.background,
    this.splashColor = Colors.black,
    this.elevation = 0,
    this.shadowColor,
    this.borderRadius,
    this.border,
    this.isInkWell = false,
    required this.onTap,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    DefaultTextStyle style = DefaultTextStyle.of(context);
    Widget content = Container(
      height: height,
      width: width,
      padding: padding,
      alignment: Alignment.center,
      decoration: BoxDecoration(
        borderRadius: borderRadius,
        border: border,
      ),
      clipBehavior: Clip.antiAlias,
      child: DefaultTextStyle(
        style: style.style.copyWith(fontFamily: "PingFang SC"),
        textAlign: style.textAlign,
        softWrap: style.softWrap,
        overflow: style.overflow,
        maxLines: style.maxLines,
        textWidthBasis: style.textWidthBasis,
        child: child,
      ),
    );
    if (isInkWell) {
      content = InkWell(
        highlightColor: splashColor.withOpacity(0.04),
        overlayColor: MaterialStateProperty.all(splashColor.withOpacity(0.12)),
        borderRadius: borderRadius,
        onTap: onTap,
        child: content,
      );
    } else {
      content = GestureDetector(
        onTap: onTap,
        child: content,
      );
    }
    return Material(
      textStyle: TextStyle(fontFamily: "PingFang SC"),
      color: background,
      borderRadius: borderRadius,
      shadowColor: shadowColor,
      elevation: elevation,
      animationDuration: Duration.zero,
      child: content,
    );
  }
}

/// TabBar 扩展组件
///
/// 自定义指示器
/// [SmartImageTabIndicator] Picture indicator
/// [SmartUnderlineTabIndicator] Underline indicator
///
class SmartTabLayout extends StatefulWidget {
  SmartTabLayout({
    this.controller,
    ScrollController? scrollController,
    this.height,
    this.width,
    this.space,
    this.spaceBuilder,
    this.minHeight = 0,
    this.minWidth = 0,
    this.margin,
    this.padding = EdgeInsets.zero,
    this.background,
    this.borderRadius,
    this.border,
    this.direction = Axis.horizontal,
    int? childCount,
    this.labels,
    this.physics,
    this.onTap,
    this.itemBuilder,
    this.isScrollable = false,
    this.itemHeight,
    this.itemWidth,
    this.itemPadding,
    this.itemBackground = Colors.transparent,
    this.itemBorderRadius,
    this.itemBorder,
    this.itemTextStyle = const TextStyle(
      color: Colors.black,
      fontSize: 14,
      fontWeight: FontWeight.w500,
    ),
    double? itemSelectedHeight,
    double? itemSelectedWidth,
    EdgeInsets? itemSelectedPadding,
    Color? itemSelectedBackground,
    BorderRadius? itemSelectedBorderRadius,
    BoxBorder? itemSelectedBorder,
    TextStyle? itemSelectedTextStyle,
    this.isInkWell = false,
    this.itemSplashColor = Colors.black,
    this.itemShadowColor,
    this.itemElevation = 0,
    this.itemTransition = false,
    this.indicatorColor,
    this.indicatorWeight = 2.0,
    this.indicatorWidth,
    this.indicatorPadding = EdgeInsets.zero,
    this.indicator,
    this.automaticIndicatorColorAdjustment = true,
    this.indicatorSize,
    this.indicatorBorderRadius = BorderRadius.zero,
  })  : assert(labels != null || childCount != null,
            "childCount and labels have different lengths"),
        assert(labels != null || itemBuilder != null,
            "labels or itemBuilder Pass in at least one"),
        this.childCount = childCount ?? labels?.length ?? 0,
        this.itemSelectedHeight = itemSelectedHeight ?? itemHeight,
        this.itemSelectedWidth = itemSelectedWidth ?? itemWidth,
        this.itemSelectedPadding = itemSelectedPadding ?? itemPadding,
        this.itemSelectedBackground = itemSelectedBackground ?? itemBackground,
        this.itemSelectedBorderRadius =
            itemSelectedBorderRadius ?? itemBorderRadius,
        this.itemSelectedBorder = itemSelectedBorder ?? itemBorder,
        this.itemSelectedTextStyle = itemSelectedTextStyle ?? itemTextStyle;

  final TabController? controller;
  final double? height;
  final double? width;
  final double? space;
  final IndexedWidgetBuilder? spaceBuilder;
  final double minHeight;
  final double minWidth;
  final EdgeInsets? margin;
  final EdgeInsets padding;
  final Color? background;
  final BorderRadius? borderRadius;
  final BoxBorder? border;
  final Axis direction;
  final int childCount;
  final List<String>? labels;
  final ScrollPhysics? physics;
  final ValueChanged<int>? onTap;
  final IndexedWidgetBuilder? itemBuilder;
  final double? itemHeight;
  final double? itemWidth;
  final EdgeInsets? itemPadding;
  final Color? itemBackground;
  final BorderRadius? itemBorderRadius;
  final BoxBorder? itemBorder;
  final TextStyle itemTextStyle;
  final double? itemSelectedHeight;
  final double? itemSelectedWidth;
  final EdgeInsets? itemSelectedPadding;
  final Color? itemSelectedBackground;
  final BorderRadius? itemSelectedBorderRadius;
  final BoxBorder? itemSelectedBorder;
  final TextStyle itemSelectedTextStyle;
  final bool isInkWell;
  final Color itemSplashColor;
  final Color? itemShadowColor;
  final double itemElevation;

  /// This parameter is used to enable TabStyle transitions.
  /// The default value is true
  final bool itemTransition;

  /// The color of the line that appears below the selected tab.
  ///
  /// If this parameter is null, then the value of the Theme's indicatorColor
  /// property is used.
  ///
  /// If [indicator] is specified or provided from [TabBarTheme],
  /// this property is ignored.
  final Color? indicatorColor;

  /// The thickness of the line that appears below the selected tab.
  ///
  /// The value of this parameter must be greater than zero and its default
  /// value is 2.0.
  ///
  /// If [indicator] is specified or provided from [TabBarTheme],
  /// this property is ignored.
  final double indicatorWeight;

  final double? indicatorWidth;

  final BorderRadius indicatorBorderRadius;

  /// Padding for indicator.
  /// This property will now no longer be ignored even if indicator is declared
  /// or provided by [TabBarTheme]
  ///
  /// For [isScrollable] tab bars, specifying [kTabLabelPadding] will align
  /// the indicator with the tab's text for [Tab] widgets and all but the
  /// shortest [Tab.text] values.
  ///
  /// The default value of [indicatorPadding] is [EdgeInsets.zero].
  final EdgeInsetsGeometry indicatorPadding;

  /// Defines the appearance of the selected tab indicator.
  ///
  /// If [indicator] is specified or provided from [TabBarTheme],
  /// the [indicatorColor], and [indicatorWeight] properties are ignored.
  ///
  /// The default, underline-style, selected tab indicator can be defined with
  /// [UnderlineTabIndicator].
  ///
  /// The indicator's size is based on the tab's bounds. If [indicatorSize]
  /// is [TabBarIndicatorSize.tab] the tab's bounds are as wide as the space
  /// occupied by the tab in the tab bar. If [indicatorSize] is
  /// [TabBarIndicatorSize.label], then the tab's bounds are only as wide as
  /// the tab widget itself.
  final Decoration? indicator;

  /// Whether this tab bar should automatically adjust the [indicatorColor].
  ///
  /// If [automaticIndicatorColorAdjustment] is true,
  /// then the [indicatorColor] will be automatically adjusted to [Colors.white]
  /// when the [indicatorColor] is same as [Material.color] of the [Material] parent widget.
  final bool automaticIndicatorColorAdjustment;

  /// Defines how the selected tab indicator's size is computed.
  ///
  /// The size of the selected tab indicator is defined relative to the
  /// tab's overall bounds if [indicatorSize] is [TabBarIndicatorSize.tab]
  /// (the default) or relative to the bounds of the tab's widget if
  /// [indicatorSize] is [TabBarIndicatorSize.label].
  ///
  /// The selected tab's location appearance can be refined further with
  /// the [indicatorColor], [indicatorWeight], [indicatorPadding], and
  /// [indicator] properties.
  final TabBarIndicatorSize? indicatorSize;

  /// Whether this tab bar can be scrolled horizontally.
  ///
  /// If [isScrollable] is true, then each tab is as wide as needed for its label
  /// and the entire [TabBar] is scrollable. Otherwise each tab gets an equal
  /// share of the available space.
  final bool isScrollable;

  @override
  State<StatefulWidget> createState() {
    return _SmartTabLayoutState();
  }
}

class _SmartTabLayoutState extends State<SmartTabLayout>
    with SingleTickerProviderStateMixin {
  List<Widget>? tabs;
  List<GlobalKey> _tabKeys = [];
  late double _tabStripWidth;
  int? _currentIndex;
  TabController? _controller;
  _IndicatorPainter? _indicatorPainter;
  ScrollController? _scrollController;

  bool get _controllerIsValid => _controller?.animation != null;

  int get maxTabIndex => _indicatorPainter!.maxTabIndex;

  Decoration get _indicator {
    if (widget.indicator != null) return widget.indicator!;
    final TabBarTheme tabBarTheme = TabBarTheme.of(context);
    if (tabBarTheme.indicator != null) return tabBarTheme.indicator!;

    Color color = widget.indicatorColor ?? Theme.of(context).indicatorColor;
    // ThemeData tries to avoid this by having indicatorColor avoid being the
    // primaryColor. However, it's possible that the tab bar is on a
    // Material that isn't the primaryColor. In that case, if the indicator
    // color ends up matching the material's color, then this overrides it.
    // When that happens, automatic transitions of the theme will likely look
    // ugly as the indicator color suddenly snaps to white at one end, but it's
    // not clear how to avoid that any further.
    //
    // The material's color might be null (if it's a transparency). In that case
    // there's no good way for us to find out what the color is so we don't.
    //
    // TODO(xu-baolin): Remove automatic adjustment to white color indicator
    // with a better long-term solution.
    // https://github.com/flutter/flutter/pull/68171#pullrequestreview-517753917
    if (widget.automaticIndicatorColorAdjustment &&
        color.value == Material.of(context)?.color?.value) color = Colors.white;

    return SmartUnderlineTabIndicator(
      direction: widget.direction,
      borderRadius: widget.indicatorBorderRadius,
      borderSide: BorderSide(
        width: widget.indicatorWeight,
        color: color,
      ),
      width: widget.indicatorWidth,
    );
  }

  @override
  void initState() {
    super.initState();
    _tabKeys.clear();
    for (int i = 0; i < widget.childCount; i++) {
      _tabKeys.add(GlobalKey());
    }
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _updateTabController();
    _initIndicatorPainter();
  }

  @override
  void didUpdateWidget(SmartTabLayout oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.controller != oldWidget.controller) {
      _updateTabController();
      _initIndicatorPainter();
    } else if (widget.indicatorColor != oldWidget.indicatorColor ||
        widget.indicatorWeight != oldWidget.indicatorWeight ||
        widget.indicatorWidth != oldWidget.indicatorWidth ||
        widget.indicatorSize != oldWidget.indicatorSize ||
        widget.indicatorPadding != oldWidget.indicatorPadding ||
        widget.indicatorBorderRadius != oldWidget.indicatorBorderRadius ||
        widget.indicator != oldWidget.indicator) {
      _initIndicatorPainter();
    }

    if (widget.childCount > oldWidget.childCount) {
      final int delta = widget.childCount - oldWidget.childCount;
      _tabKeys.addAll(List<GlobalKey>.generate(delta, (int n) => GlobalKey()));
    } else if (widget.childCount < oldWidget.childCount) {
      _tabKeys.removeRange(widget.childCount, oldWidget.childCount);
    }
  }

  void _updateTabController() {
    final TabController? newController =
        widget.controller ?? DefaultTabController.of(context);
    assert(() {
      if (newController == null) {
        throw FlutterError(
          'No TabController for ${widget.runtimeType}.\n'
          'When creating a ${widget.runtimeType}, you must either provide an explicit '
          'TabController using the "controller" property, or you must ensure that there '
          'is a DefaultTabController above the ${widget.runtimeType}.\n'
          'In this case, there was neither an explicit controller nor a default controller.',
        );
      }
      return true;
    }());

    if (newController == _controller) return;

    assert((newController?.length) == widget.childCount,
        "childCount and controller.length have different lengths");

    if (_controllerIsValid) {
      _controller!.animation!.removeListener(_handleTabControllerAnimationTick);
      _controller!.removeListener(_handleTabControllerTick);
    }
    _controller = newController;
    if (_controller != null) {
      _controller!.animation!.addListener(_handleTabControllerAnimationTick);
      _controller!.addListener(_handleTabControllerTick);
      _currentIndex = _controller!.index;
    }
  }

  void _initIndicatorPainter() {
    _indicatorPainter = !_controllerIsValid
        ? null
        : _IndicatorPainter(
            controller: _controller!,
            direction: widget.direction,
            indicator: _indicator,
            indicatorSize:
                widget.indicatorSize ?? TabBarTheme.of(context).indicatorSize,
            indicatorPadding: widget.indicatorPadding,
            indicatorWidth: widget.indicatorWidth,
            tabKeys: _tabKeys,
            old: _indicatorPainter,
          );
  }

  @override
  Widget build(BuildContext context) {
    tabs = _buildTab();

    // 文字过渡
    if (_controller != null) {
      if (widget.itemTransition) {
        if (_controller!.indexIsChanging) {
          // The user tapped on a tab, the tab controller's animation is running.
          final Animation<double> animation = _ChangeAnimation(_controller!);
          int previousIndex = _controller!.previousIndex;
          tabs![_currentIndex!] = _buildAnimStyledTab(
              tabs![_currentIndex!], true, _currentIndex!, animation);
          tabs![previousIndex] = _buildAnimStyledTab(
              tabs![previousIndex], false, previousIndex, animation);
          for (int i = 0; i < tabs!.length; i++) {
            if (i != _currentIndex && i != previousIndex) {
              tabs![i] = _buildStyledTab(tabs![i], i);
            }
          }
        } else {
          final int tabIndex = _currentIndex!;
          final Animation<double> centerAnimation =
              _DragAnimation(_controller!, tabIndex);
          tabs![tabIndex] = _buildAnimStyledTab(
              tabs![tabIndex], true, tabIndex, centerAnimation);

          // if (tabIndex > 0) {
          //   final int tabIndex = _currentIndex! - 1;
          //   final Animation<double> previousAnimation =
          //       ReverseAnimation(_DragAnimation(_controller!, tabIndex));
          //   tabs![tabIndex] = _buildAnimStyledTab(
          //       tabs![tabIndex], false, tabIndex, previousAnimation);
          // }
          // if (tabIndex < (tabs?.length ?? 0) - 1) {
          //   final int tabIndex = _currentIndex! + 1;
          //   final Animation<double> nextAnimation =
          //       ReverseAnimation(_DragAnimation(_controller!, tabIndex));
          //   tabs![tabIndex] = _buildAnimStyledTab(
          //       tabs![tabIndex], false, tabIndex, nextAnimation);
          // }
          for (int i = 0; i < tabs!.length; i++) {
            if (i != tabIndex) {
              tabs![i] = _buildStyledTab(tabs![i], i);
            }
          }
        }
      } else {
        for (int i = 0; i < tabs!.length; i++) {
          tabs![i] = _buildStyledTab(tabs![i], i);
        }
      }
    }

    if (!widget.isScrollable) {
      for (int i = 0; i < tabs!.length; i++) {
        tabs![i] = Expanded(child: tabs![i]);
      }
    }

    Widget tabLayout = Padding(
      padding: widget.padding,
      child: CustomPaint(
        painter: _indicatorPainter,
        child: _TabStyle(
          selected: false,
          animation: kAlwaysDismissedAnimation,
          labelStyle: widget.itemTextStyle,
          selectedLabelStyle: widget.itemSelectedTextStyle,
          child: _TabLabelBar(
            onPerformLayout: _saveTabOffsets,
            direction: widget.direction,
            children: _insetSpace(),
          ),
        ),
      ),
    );
    if (widget.isScrollable) {
      _scrollController = _TabBarScrollController(this);
      tabLayout = SingleChildScrollView(
        physics: widget.physics,
        controller: _scrollController,
        scrollDirection: widget.direction,
        child: tabLayout,
      );
    }

    return Container(
      height: widget.height,
      width: widget.width,
      margin: widget.margin,
      constraints: BoxConstraints(
        minHeight: widget.minHeight,
        minWidth: widget.minWidth,
      ),
      decoration: BoxDecoration(
        color: widget.background,
        borderRadius: widget.borderRadius,
        border: widget.border,
      ),
      child: tabLayout,
    );
  }

  List<Widget> _buildTab() {
    List<Widget> tabs = [];
    for (int i = 0; i < widget.childCount; i++) {
      Widget child = widget.itemBuilder?.call(context, i) ??
          Center(child: Text(widget.labels![i]));
      tabs.add(KeyedSubtree(key: _tabKeys[i], child: child));
    }
    return tabs;
  }

  _insetSpace() {
    List<Widget> list = [];
    for (int i = 0; i < widget.childCount; i++) {
      list.add(tabs![i]);
      if (i != widget.childCount - 1) {
        if (widget.direction == Axis.horizontal) {
          list.add(widget.spaceBuilder?.call(context, i) ??
              SizedBox(width: widget.space));
        } else {
          list.add(widget.spaceBuilder?.call(context, i) ??
              SizedBox(height: widget.space));
        }
      }
    }
    return list;
  }

  _buildStyledTab(Widget child, int index) {
    bool selected = index == _controller?.index;
    double? width = selected ? widget.itemSelectedWidth : widget.itemWidth;
    double? height = selected ? widget.itemSelectedHeight : widget.itemHeight;
    EdgeInsets? padding =
        selected ? widget.itemSelectedPadding : widget.itemPadding;
    Color? background =
        selected ? widget.itemSelectedBackground : widget.itemBackground;
    BoxBorder? border =
        selected ? widget.itemSelectedBorder : widget.itemBorder;
    BorderRadius? borderRadius =
        selected ? widget.itemSelectedBorderRadius : widget.itemBorderRadius;
    return _TabStyle(
      selected: selected,
      animation: kAlwaysDismissedAnimation,
      labelStyle: widget.itemTextStyle,
      selectedLabelStyle: widget.itemSelectedTextStyle,
      child: SmartTab(
        width: width,
        height: height,
        padding: padding,
        background: background,
        isInkWell: widget.isInkWell,
        splashColor: widget.itemSplashColor,
        shadowColor: widget.itemShadowColor,
        elevation: widget.itemElevation,
        border: border,
        borderRadius: borderRadius,
        child: child,
        onTap: () {
          _controller!.animateTo(index);
          widget.onTap?.call(index);
        },
      ),
    );
  }

  Widget _buildAnimStyledTab(
      Widget child, bool selected, int index, Animation<double> animation) {
    return _TabStyle(
      selected: selected,
      animation: _ChangeAnimation(_controller!),
      labelStyle: widget.itemTextStyle,
      selectedLabelStyle: widget.itemSelectedTextStyle,
      child: child,
      index: index,
      widget: widget,
    );
  }

  // Called each time layout completes.
  void _saveTabOffsets(
      List<double> tabOffsets, TextDirection textDirection, double width) {
    _tabStripWidth = width;
    _indicatorPainter?.saveTabOffsets(tabOffsets, textDirection);
  }

  void _handleTabControllerTick() {
    if (_controller!.index != _currentIndex) {
      _currentIndex = _controller!.index;
      if (widget.isScrollable) _scrollToCurrentIndex();
    }
    setState(() {
      // Rebuild the tabs after a (potentially animated) index change
      // has completed.
    });
  }

  void _handleTabControllerAnimationTick() {
    assert(mounted);
    if (!_controller!.indexIsChanging && widget.isScrollable) {
      _scrollToControllerValue();
    }
  }

  /// 滚动修正
  void _scrollToCurrentIndex() {
    final double offset = _tabCenteredScrollOffset(_currentIndex!);
    _scrollController!
        .animateTo(offset, duration: kTabScrollDuration, curve: Curves.ease);
  }

  void _scrollToControllerValue() {
    final double? leadingPosition = _currentIndex! > 0
        ? _tabCenteredScrollOffset(_currentIndex! - 1)
        : null;
    final double middlePosition = _tabCenteredScrollOffset(_currentIndex!);
    final double? trailingPosition = _currentIndex! < maxTabIndex
        ? _tabCenteredScrollOffset(_currentIndex! + 1)
        : null;

    final double index = _controller!.index.toDouble();
    final double value = _controller!.animation!.value;
    final double offset;
    if (value == index - 1.0)
      offset = leadingPosition ?? middlePosition;
    else if (value == index + 1.0)
      offset = trailingPosition ?? middlePosition;
    else if (value == index)
      offset = middlePosition;
    else if (value < index)
      offset = leadingPosition == null
          ? middlePosition
          : lerpDouble(middlePosition, leadingPosition, index - value)!;
    else
      offset = trailingPosition == null
          ? middlePosition
          : lerpDouble(middlePosition, trailingPosition, value - index)!;

    _scrollController!.jumpTo(offset);
  }

  double _tabScrollOffset(
      int index, double viewportWidth, double minExtent, double maxExtent) {
    if (!widget.isScrollable) return 0.0;
    double tabCenter = _indicatorPainter!.centerOf(index);
    switch (Directionality.of(context)) {
      case TextDirection.rtl:
        tabCenter = _tabStripWidth - tabCenter;
        break;
      case TextDirection.ltr:
        break;
    }
    return (tabCenter - viewportWidth / 2.0).clamp(minExtent, maxExtent);
  }

  double _tabCenteredScrollOffset(int index) {
    final ScrollPosition position = _scrollController!.position;
    return _tabScrollOffset(index, position.viewportDimension,
        position.minScrollExtent, position.maxScrollExtent);
  }

  double _initialScrollOffset(
      double viewportWidth, double minExtent, double maxExtent) {
    return _tabScrollOffset(
        _currentIndex!, viewportWidth, minExtent, maxExtent);
  }

  @override
  void dispose() {
    _indicatorPainter?.dispose();
    if (_controllerIsValid) {
      _controller!.animation!.removeListener(_handleTabControllerAnimationTick);
      _controller!.removeListener(_handleTabControllerTick);
    }
    _controller = null;
    // We don't own the _controller Animation, so it's not disposed here.
    super.dispose();
  }
}

/// 字体过渡动画
class _ChangeAnimation extends Animation<double>
    with AnimationWithParentMixin<double> {
  _ChangeAnimation(this.controller);

  final TabController controller;

  @override
  Animation<double> get parent => controller.animation!;

  @override
  void removeStatusListener(AnimationStatusListener listener) {
    if (controller.animation != null) super.removeStatusListener(listener);
  }

  @override
  void removeListener(VoidCallback listener) {
    if (controller.animation != null) super.removeListener(listener);
  }

  @override
  double get value => _indexChangeProgress(controller);

  double _indexChangeProgress(TabController controller) {
    final double controllerValue = controller.animation!.value;
    final double previousIndex = controller.previousIndex.toDouble();
    final double currentIndex = controller.index.toDouble();

    // The controller's offset is changing because the user is dragging the
    // TabBarView's PageView to the left or right.
    if (!controller.indexIsChanging)
      return (currentIndex - controllerValue).abs().clamp(0.0, 1.0);

    // The TabController animation's value is changing from previousIndex to currentIndex.
    return (controllerValue - currentIndex).abs() /
        (currentIndex - previousIndex).abs();
  }
}

class _DragAnimation extends Animation<double>
    with AnimationWithParentMixin<double> {
  _DragAnimation(this.controller, this.index);

  final TabController controller;
  final int index;

  @override
  Animation<double> get parent => controller.animation!;

  @override
  void removeStatusListener(AnimationStatusListener listener) {
    if (controller.animation != null) super.removeStatusListener(listener);
  }

  @override
  void removeListener(VoidCallback listener) {
    if (controller.animation != null) super.removeListener(listener);
  }

  @override
  double get value {
    assert(!controller.indexIsChanging);
    final double controllerMaxValue = (controller.length - 1).toDouble();
    final double controllerValue =
        controller.animation!.value.clamp(0.0, controllerMaxValue);
    return (controllerValue - index.toDouble()).abs().clamp(0.0, 1.0);
  }
}
