import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:smart_tab_layout/src/smart_image_tab_indicator.dart';

/// Used with [TabBar.indicator] to draw a horizontal line below the
/// selected tab.
///
/// The selected tab underline is inset from the tab's boundary by [insets].
/// The [borderSide] defines the line's color and weight.
///
/// The [TabBar.indicatorSize] property can be used to define the indicator's
/// bounds in terms of its (centered) widget with [TabBarIndicatorSize.label],
/// or the entire tab with [TabBarIndicatorSize.tab].
///
/// [SmartImageTabIndicator] Picture indicator
/// [SmartUnderlineTabIndicator] Underline indicator
///
class SmartUnderlineTabIndicator extends Decoration {
  /// Create an underline style selected tab indicator.
  ///
  /// The [borderSide] and [insets] arguments must not be null.
  const SmartUnderlineTabIndicator({
    this.borderRadius = BorderRadius.zero,
    this.direction = Axis.horizontal,
    this.borderSide = const BorderSide(width: 2.0, color: Colors.white),
    this.width,
    this.insets = EdgeInsets.zero,
  })  : assert(borderSide != null),
        assert(insets != null);

  final BorderRadius borderRadius;

  final Axis direction;

  /// The color and weight of the horizontal line drawn below the selected tab.
  final BorderSide borderSide;

  final double? width;

  /// Locates the selected tab's underline relative to the tab's boundary.
  ///
  /// The [TabBar.indicatorSize] property can be used to define the tab
  /// indicator's bounds in terms of its (centered) tab widget with
  /// [TabBarIndicatorSize.label], or the entire tab with
  /// [TabBarIndicatorSize.tab].
  final EdgeInsetsGeometry insets;

  @override
  Decoration? lerpFrom(Decoration? a, double t) {
    if (a is UnderlineTabIndicator) {
      return SmartUnderlineTabIndicator(
        borderSide: BorderSide.lerp(a.borderSide, borderSide, t),
        insets: EdgeInsetsGeometry.lerp(a.insets, insets, t)!,
      );
    }
    return super.lerpFrom(a, t);
  }

  @override
  Decoration? lerpTo(Decoration? b, double t) {
    if (b is UnderlineTabIndicator) {
      return SmartUnderlineTabIndicator(
        borderSide: BorderSide.lerp(borderSide, b.borderSide, t),
        insets: EdgeInsetsGeometry.lerp(insets, b.insets, t)!,
      );
    }
    return super.lerpTo(b, t);
  }

  @override
  _UnderlinePainter createBoxPainter([VoidCallback? onChanged]) {
    return _UnderlinePainter(this, borderRadius, direction, onChanged);
  }

  Rect _indicatorRectFor(Rect rect, TextDirection textDirection) {
    assert(rect != null);
    assert(textDirection != null);
    final Rect indicator = insets.resolve(textDirection).deflateRect(rect);

    final width;
    final left;
    if (direction == Axis.horizontal) {
      if (this.width == null) {
        width = indicator.width;
        left = indicator.left;
      } else {
        width = this.width!;
        left = indicator.center.dx - (this.width! / 2);
      }
      return Rect.fromLTWH(
        left,
        indicator.bottom - borderSide.width,
        width,
        borderSide.width,
      );
    } else {
      if (this.width == null) {
        width = indicator.height;
        left = indicator.top;
      } else {
        width = this.width!;
        left = indicator.center.dy - (this.width! / 2);
      }
      return Rect.fromLTWH(
        indicator.right - borderSide.width,
        left,
        borderSide.width,
        width,
      );
    }
  }

  @override
  Path getClipPath(Rect rect, TextDirection textDirection) {
    return Path()..addRect(_indicatorRectFor(rect, textDirection));
  }
}

class _UnderlinePainter extends BoxPainter {
  _UnderlinePainter(this.decoration, this.borderRadius, this.direction,
      VoidCallback? onChanged)
      : assert(decoration != null),
        super(onChanged);

  final BorderRadius borderRadius;
  final Axis direction;
  final SmartUnderlineTabIndicator decoration;

  @override
  void paint(Canvas canvas, Offset offset, ImageConfiguration configuration) {
    assert(configuration != null);
    assert(configuration.size != null);
    final Rect rect = offset & configuration.size!;
    final TextDirection textDirection = configuration.textDirection!;
    final Rect indicator = decoration._indicatorRectFor(rect, textDirection);
    final Paint paint = decoration.borderSide.toPaint()
      ..style = PaintingStyle.fill;
    canvas.drawRRect(
        RRect.fromRectAndCorners(
          indicator,
          topRight: borderRadius.topRight,
          topLeft: borderRadius.topLeft,
          bottomLeft: borderRadius.bottomLeft,
          bottomRight: borderRadius.bottomRight,
        ),
        paint);
  }
}
